﻿Imports System.Collections.ObjectModel
Imports ThinkGeo.MapSuite.Drawing
Imports ThinkGeo.MapSuite.Layers
Imports ThinkGeo.MapSuite.Shapes
Imports ThinkGeo.MapSuite.Styles

Public Class CustomLabelPrinterLayer
  Inherits LabelPrinterLayer
  'Private oWrappedText As List(Of StringData)
  Public Sub New()
    MyBase.New
  End Sub
  Public Sub New(sText As String, oFont As GeoFont, oBrush As GeoBrush)
    MyBase.New(sText, oFont, oBrush)
  End Sub
  Protected Overrides Sub DrawCore(canvas As GeoCanvas, labelsInAllLayers As Collection(Of SimpleCandidate))
    If String.IsNullOrEmpty(MyBase.Text) Then
      Return
    End If

    If PrinterWrapMode = PrinterWrapMode.WrapText Then
      Dim oneToOneScale As Double = PrinterHelper.GetPointsPerGeographyUnit(canvas.MapUnit)
      Dim drawingSize As Single = CSng(Font.Size * oneToOneScale / canvas.CurrentScale)
      Dim drawingFont As GeoFont = New GeoFont(Font.FontName, drawingSize, Font.Style)

      Dim dX As Double
      Dim dY As Double

      Dim oCurrentBoundingBox As RectangleShape = GetBoundingBox()

      Debug.WriteLine("BBox Width: " + oCurrentBoundingBox.Width.ToString)
      Dim oWrappedText = mGetWrappedText(canvas, Text, oCurrentBoundingBox.Width, Font)

      Dim dYChange As Double = PrinterHelper.ConvertLength(0.12, PrintingUnit.Inch, PrintingUnit.Point)
      dYChange = canvas.MeasureText(Text, Font).Height
      Dim iCounter As Integer = 0
      For Each oStringData In oWrappedText
        Dim oTextRect = canvas.MeasureText(oStringData, Font)

        dX = oCurrentBoundingBox.GetCenterPoint().X - oCurrentBoundingBox.Width / 2 + oTextRect.Width / 2
        dY = oCurrentBoundingBox.UpperLeftPoint.Y - oTextRect.Height / 2
        canvas.DrawTextWithWorldCoordinate(oStringData, drawingFont, TextBrush, dX, dY - (iCounter * dYChange), DrawingLevel.LabelLevel)
        iCounter += 1
      Next oStringData
      If Me.BackgroundMask IsNot Nothing Then
        canvas.DrawArea(Me.GetBoundingBox, BackgroundMask.OutlinePen, BackgroundMask.FillSolidBrush, DrawingLevel.LevelOne)
      End If
    Else
      MyBase.DrawCore(canvas, labelsInAllLayers)
    End If
  End Sub
  'Private Function mGetWrappedText(oCanvas As GeoCanvas, sText As String, dTextWidth As Double, oFont As GeoFont) As List(Of String)
  '  Dim oWrappedText As New List(Of String)
  '  Dim dLineLength As Double = 0.0
  '  Dim oTextLine As New Text.StringBuilder
  '  Dim iWordsInList As Integer
  '  Dim iTotalWords As Integer
  '  Dim iCurrentWordCount As Integer

  '  '***** Divide the provided string up into strings that are less than or equal to the available width
  '  Dim oWords() As String = sText.Split(" "c)
  '  iTotalWords = oWords.Count
  '  For Each s As String In oWords
  '    If s = String.Empty Then
  '      Continue For
  '    End If
  '    dLineLength += oCanvas.MeasureText(s, oFont).Width
  '    If dLineLength > dTextWidth Then
  '      If oTextLine.Length > 0 Then
  '        oTextLine.Remove(oTextLine.Length - 1, 1)
  '        oWrappedText.Add(oTextLine.ToString)
  '        oTextLine.Clear()
  '      End If
  '      oTextLine.Append(s + " ")
  '      iWordsInList += iCurrentWordCount
  '      iCurrentWordCount = 1
  '      dLineLength = oCanvas.MeasureText(s, oFont).Width
  '    Else
  '      oTextLine.Append(s + " ")
  '      iCurrentWordCount += 1
  '    End If
  '  Next s

  '  If iWordsInList < iTotalWords Then
  '    oTextLine.Remove(oTextLine.Length - 1, 1)
  '    oWrappedText.Add(oTextLine.ToString)
  '  End If

  '  Return oWrappedText

  'End Function
  Private Function mGetWrappedText(oCanvas As GeoCanvas, sText As String, dTextWidth As Double, oFont As GeoFont) As List(Of String)
    Dim oWrappedText As New List(Of String)
    Dim dLineLength As Double = 0.0
    Dim oTextLine As New Text.StringBuilder
    Dim iWordsInList As Integer
    Dim iTotalWords As Integer
    Dim iCurrentWordCount As Integer

    '***** Divide the provided string up into strings that are less than or equal to the available width
    Dim oWords() As String = sText.Split(" "c)
    iTotalWords = oWords.Count
    For Each s As String In oWords
      If s = String.Empty Then
        Continue For
      End If
      dLineLength += oCanvas.MeasureText(s, oFont).Width
      If dLineLength > dTextWidth Then
        If oTextLine.Length > 0 Then
          oTextLine.Remove(oTextLine.Length - 1, 1)
          'Dim oStringData As New StringData() With {.Text = oTextLine.ToString}
          oWrappedText.Add(oTextLine.ToString)
          oTextLine.Clear()
        End If
        oTextLine.Append(s + " ")
        iWordsInList += iCurrentWordCount
        iCurrentWordCount = 1
        dLineLength = oCanvas.MeasureText(s, oFont).Width
        'Debug.WriteLine("Word: " + s + ", Text length: " + dLineLength.ToString)
      Else
        oTextLine.Append(s + " ")
        iCurrentWordCount += 1
        'Debug.WriteLine("Word: " + s + ", Text length: " + dLineLength.ToString)
      End If
    Next s

    If iWordsInList < iTotalWords Then
      oTextLine.Remove(oTextLine.Length - 1, 1)
      'Dim oStringData As New StringData() With {.Text = oTextLine.ToString}
      oWrappedText.Add(oTextLine.ToString)
    End If

    For Each s In oWrappedText
      Debug.WriteLine("Line: " + s + ", Width: " + oCanvas.MeasureText(s, oFont).Width.ToString)
    Next

    Return oWrappedText

  End Function

End Class
